#-----------------------------------------------------------------------------
#
#   Loon - Unit command menu widget
#
#-----------------------------------------------------------------------------

from pygame import Color
from albow.utils import blit_in_rect
from albow.palette_view import PaletteView
from action_widget import ActionWidget
from game import game

class UnitCommandMenu(PaletteView, ActionWidget):

	button_size = (96, 32)
	bg_color = Color("lightblue")
	highlight_color = Color("orange")
	clicked_item = None
	clicked_rect = None
	in_clicked_item = False
	
	@property
	def world(self):
		return game.world
	
	@property
	def valid_commands(self):
		return self.world.valid_unit_commands

	def __init__(self, client):
		PaletteView.__init__(self, self.button_size, 3, 3)
		self.client = client
	
	def num_items(self):
		return len(self.valid_commands)
	
	def draw_item(self, surf, i, rect):
		if i == self.clicked_item and self.in_clicked_item:
			surf.fill(self.highlight_color, rect)
		cmd = self.valid_commands[i]
		text = cmd.title
		key = cmd.key_name
		if key:
			text = "%s (%s)" % (text, key)
		buf = self.font.render(text, True, self.fg_color)
		blit_in_rect(surf, buf, rect, 'c')

	def click_item(self, i, e):
		self.clicked_item = i
		self.clicked_rect = self.item_rect(i)
		self.in_clicked_item = True
	
	def mouse_drag(self, e):
		r = self.clicked_rect
		h = r is not None and r.collidepoint(e.local)
		if self.in_clicked_item <> h:
			self.in_clicked_item = h
			self.invalidate()

	def mouse_up(self, e):
		if self.in_clicked_item:
			cmd = self.valid_commands[self.clicked_item]
			self.invoke_command(cmd)
		self.clicked_item = None
		self.clicked_rect = None
		self.in_clicked_item = False
	
	def invoke_command(self, cmd):
		self.client.set_current_command(cmd)
		cmd.invoke(self.client)

	def item_rect(self, i):
		nr = self.num_rows()
		return self.cell_rect(i // nr, i % nr)

	def world_click(self, e):
		self.client.perform_unit_selection(e)

	def key_down(self, e):
		key = e.key
		for cmd in self.valid_commands:
			if cmd.key == key:
				self.invoke_command(cmd)
				return
		PaletteView.key_down(self, e)
