#-----------------------------------------------------------------------------
#
#   Loon - Unit commands
#
#-----------------------------------------------------------------------------

from numpy import ceil, rint
from pygame.locals import *
from vector_algebra import vector
import missions
import resources

unit_commands_by_name = {}

#-----------------------------------------------------------------------------

class UnitCommand(object):

	key = None
	key_name = None

	def __init__(self, name, title):
		self.name = name
		self.title = title
		unit_commands_by_name[name] = self
	
#-----------------------------------------------------------------------------

class TargetedCommand(UnitCommand):

	def invoke(self, client):
		client.enter_target_selection_mode(self)

#-----------------------------------------------------------------------------

class MoveCommand(TargetedCommand):

	key = K_m
	key_name = "M"

	def invoke_on_unit_with_target(self, unit, target):
		unit.assign_mission(missions.MoveMission(target))

#-----------------------------------------------------------------------------

class MineCommand(TargetedCommand):

	key = K_d
	key_name = "D"
	
	def invoke_on_unit_with_target(self, unit, target):
		unit.assign_mission(missions.MineMission(target))

#-----------------------------------------------------------------------------

#class AttackCommand(TargetedCommand):
#
#	key = K_a
#	key_name = "A"
#
#	def invoke_on_unit_with_target(self, unit, target):
#		unit.perform_attack_cmd(target)

#-----------------------------------------------------------------------------

class BuildCommand(UnitCommand):

	key = K_b
	key_name = "B"

	def invoke(self, client):
		client.enter_building_selection_mode(self)
		
	def invoke_on_unit_with_site_and_class(self, client, unit, p, unit_class):
		print "BuildCommand.invoke_on_unit_with_site_and_class: p =", p ###
		building_site = client.world.site_suitable_for_unit_class(p, unit_class)
		if building_site is not None:
			target = building_site
			missions.BuildMission(target, unit, unit_class).begin(unit)
			client.enter_unit_command_mode()

	def required_site_size(self, unit_class):
		s = int(ceil(2 * unit_class.radius))
		return vector(s, s)

#-----------------------------------------------------------------------------

class WorkCommand(TargetedCommand):

	key = K_w
	key_name = "W"
	
	def invoke_on_unit_with_target(self, unit, target):
		unit.assign_mission(missions.WorkMission(target))

#-----------------------------------------------------------------------------

class GetCommand(TargetedCommand):

	key = K_g
	key_name = "G"
	
	def invoke_on_unit_with_target(self, unit, target):
		unit.assign_mission(missions.GetMission(target))

#-----------------------------------------------------------------------------

class PutCommand(TargetedCommand):

	key = K_p
	key_name = "P"
	
	def invoke_on_unit_with_target(self, unit, target):
		unit.assign_mission(missions.PutMission(target))

#-----------------------------------------------------------------------------

class CheatCommand(UnitCommand):

	key = K_c
	key_name = "C"

	def invoke(self, client):
		unit = client.acting_unit
		if unit:
			for name in resources.resource_types:
				unit.resources.add(name, 100)
			client.enter_unit_command_mode()

#-----------------------------------------------------------------------------

unit_command_list = [
	MoveCommand('move', "Move"),
	#AttackCommand('attack', "Attack"),
	GetCommand('get', "Get"),
	PutCommand('put', "Put"),
	MineCommand('mine', "Mine"),
	BuildCommand('build', "Build"),
	WorkCommand('work', "Work"),
	CheatCommand('cheat', "Cheat"),
]
